import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/auth/model/project_model.dart';
import 'package:syncrow_web/pages/routines/bloc/automation_scene_trigger_bloc/automation_scene_trigger_event.dart';
import 'package:syncrow_web/pages/routines/bloc/automation_scene_trigger_bloc/automation_scene_trigger_status.dart';
import 'package:syncrow_web/pages/routines/models/routine_model.dart';
import 'package:syncrow_web/services/routines_api.dart';

class AutomationSceneTriggerBloc extends Bloc<AutomationSceneTriggerEvent, AutomationSceneTriggerStatus> {
  AutomationSceneTriggerBloc() : super(AutomationSceneInitial()) {
    // on<LoadScenes>(_onLoadScenes);
    // on<LoadAutomation>(_onLoadAutomation);
    on<SceneTrigger>(_onSceneTrigger);
    on<UpdateAutomationStatus>(_onUpdateAutomationStatus);
  }

  List<ScenesModel> scenes = [];
  List<ScenesModel> automationList = [];

  // Future<void> _onLoadScenes(LoadScenes event, Emitter<AutomationSceneTriggerStatus> emit) async {
  //   emit(SceneLoading());

  //   try {
  //     Project? project = HomeCubit.getInstance().project;

  //     if (event.unitId.isNotEmpty) {
  //       scenes = await SceneApi.getScenesByUnitId(event.unitId,
  //           event.unit.community.uuid, project?.uuid ?? TempConst.projectIdDev,
  //           showInDevice: event.showInDevice);
  //       emit(SceneLoaded(scenes, automationList));
  //     } else {
  //       emit(const SceneError(message: 'Unit ID is empty'));
  //     }
  //   } catch (e) {
  //     emit(const SceneError(message: 'Something went wrong'));
  //   }
  // }

  // Future<void> _onLoadAutomation(
  //     LoadAutomation event, Emitter<AutomationSceneTriggerStatus> emit) async {
  //   emit(SceneLoading());

  //   try {
  //     Project? project = HomeCubit.getInstance().project;

  //     if (event.unitId.isNotEmpty) {
  //       automationList = await SceneApi.getAutomationByUnitId(
  //           event.unitId, event.communityId, project?.uuid ?? '');
  //       emit(SceneLoaded(scenes, automationList));
  //     } else {
  //       emit(const SceneError(message: 'Unit ID is empty'));
  //     }
  //   } catch (e) {
  //     emit(const SceneError(message: 'Something went wrong'));
  //   }
  // }

  Future<void> _onSceneTrigger(
      SceneTrigger event, Emitter<AutomationSceneTriggerStatus> emit) async {
    final currentState = state;
    if (currentState is AutomationSceneLoaded) {
      emit(AutomationSceneLoaded(
        currentState.scenes,
        currentState.automationList,
        loadingSceneId: event.sceneId,
      ));

      try {
        final success = await SceneApi.triggerScene(event.sceneId);
        if (success) {
          emit(SceneTriggerSuccess(event.name));
          emit(AutomationSceneLoaded(currentState.scenes, currentState.automationList));
        } else {
          emit(const AutomationSceneError(message: 'Something went wrong'));
        }
      } catch (e) {
        emit(const AutomationSceneError(message: 'Something went wrong'));
      }
    }
  }

  Future<void> _onUpdateAutomationStatus(
      UpdateAutomationStatus event, Emitter<AutomationSceneTriggerStatus> emit) async {
    final currentState = state;
    if (currentState is AutomationSceneLoaded) {
      final newLoadingStates =
          Map<String, bool>.from(currentState.loadingStates)
            ..[event.automationId] = true;

      emit(AutomationSceneLoaded(
        currentState.scenes,
        currentState.automationList,
        loadingStates: newLoadingStates,
      ));

      try {
        Project? project = HomeCubit.getInstance().project;

        final success = await SceneApi.updateAutomationStatus(
            event.automationId,
            event.automationStatusUpdate,
            project?.uuid ?? '');
        if (success) {
          automationList = await SceneApi.getAutomationByUnitId(
              event.automationStatusUpdate.spaceUuid,
              event.communityId,
              project?.uuid ?? '');
          newLoadingStates[event.automationId] = false;
          emit(AutomationSceneLoaded(
            currentState.scenes,
            automationList,
            loadingStates: newLoadingStates,
          ));
        } else {
          emit(const AutomationSceneError(message: 'Something went wrong'));
        }
      } catch (e) {
        emit(const AutomationSceneError(message: 'Something went wrong'));
      }
    }
  }
}
